/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997
 *	Sleepycat Software.  All rights reserved.
 */

#include "config.h"

#ifndef lint
static const char sccsid[] = "@(#)cxx_lock.cpp	10.3 (Sleepycat) 7/20/97";
#endif /* not lint */

#include "db_cxx.h"
#include "cxx_int.h"
#include <errno.h>

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            DBLockTable                             //
//                                                                    //
////////////////////////////////////////////////////////////////////////

DBLockTable::DBLockTable()
:   imp_(0)
{
}

DBLockTable::DBLockTable(const char *dir, int flags, int mode, DBApp &app)
:   imp_(0)
{
    DB_LOCKTAB *locktab = 0;
    int err = 0;
    if ((err = lock_open(dir, flags, mode, unwrap(&app), &locktab)) != 0) {
        DB_ERROR("DBLockTable::constructor", err);
    }
    imp_ = wrap(locktab);
}

DBLockTable::~DBLockTable()
{
    // Note: could do automatic close, but only if we reference count
}

int DBLockTable::isValid() const
{
    return (unwrapConst(this) != 0);
}

int DBLockTable::close()
{
    DB_LOCKTAB *locktab = unwrap(this);
    int err;

    if (!locktab) {
        return EINVAL;        // handle never assigned
    }

    if ((err = lock_close(locktab)) != 0) {
        DB_ERROR("DBLockTable::close", err);
        return err;
    }
    imp_ = 0;
    return err;
}

int DBLockTable::get(LockerId locker, int flags, const DBThunk &obj, LockMode lock_mode, DBLock *lock_returned)
{
    DB_LOCKTAB *locktab = unwrap(this);

    if (!locktab) {
        return EINVAL;        // handle never assigned
    }

    int err = 0;
    DB_LOCK lock;
    if ((err = lock_get(locktab, locker, flags, unwrapConst(&obj),
                        (db_lockmode_t)lock_mode, &lock)) != 0) {
        DB_ERROR("DBLockTable::get", err);
        return err;
    }
    lock_returned->imp_ = lock;
    return err;
}

int DBLockTable::id(DBLockTable::LockerId *id_returned)
{
    DB_LOCKTAB *locktab = unwrap(this);

    if (!locktab) {
        return EINVAL;        // handle never assigned
    }

    int err;
    if ((err = lock_id(locktab, id_returned)) != 0) {
        DB_ERROR("DBLockTable::id", err);
    }
    return err;
}

int DBLockTable::put(DBLock lock)
{
    DB_LOCKTAB *locktab = unwrap(this);

    if (!locktab) {
        return EINVAL;        // handle never assigned
    }

    int err;
    if ((err = lock_put(locktab, unwrap(&lock))) != 0) {
        DB_ERROR("DBLockTable::put", err);
    }
    return err;
}

int DBLockTable::vector(LockerId locker, int flags,
                         DB_LOCKREQ list[],
                         int nlist, DB_LOCKREQ **elist_returned)
{
    DB_LOCKTAB *locktab = unwrap(this);

    if (!locktab) {
        return EINVAL;        // handle never assigned
    }

    int err;
    if ((err = lock_vec(locktab, locker, flags, list,
                        nlist, elist_returned)) != 0) {
        DB_ERROR("DBLockTable::put", err);
        return err;
    }
    return err;
}

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            DBLock                                  //
//                                                                    //
////////////////////////////////////////////////////////////////////////

DBLock::DBLock(unsigned int value)
:   imp_(value)
{
}

DBLock::DBLock()
:   imp_(0)
{
}

unsigned int DBLock::getId()
{
    return imp_;
}

void DBLock::setId(unsigned int value)
{
    imp_ = value;
}
