/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 1997
 *	Sleepycat Software.  All rights reserved.
 */

#include "config.h"

#ifndef lint
static const char sccsid[] = "@(#)cxx_txn.cpp	10.3 (Sleepycat) 7/20/97";
#endif /* not lint */

#include "db_cxx.h"
#include "cxx_int.h"
#include <errno.h>

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            DBTransactionManager                    //
//                                                                    //
////////////////////////////////////////////////////////////////////////

DBTransactionManager::DBTransactionManager()
:   imp_(0)
{
}

DBTransactionManager::DBTransactionManager(const char *dir, int flags, int mode, DBApp &dbenv)
:   imp_(0)
{
    int err;
    DB_TXNMGR *mgr;
    if ((err = txn_open(dir, flags, mode, unwrap(&dbenv), &mgr)) != 0) {
        DB_ERROR("DBTransactionManager::DBTransactionManager", err);
    }
    imp_ = wrap(mgr);
}

int DBTransactionManager::close()
{
    int err;
    DB_TXNMGR *mgr = unwrap(this);
    if ((err = txn_close(mgr)) != 0) {
        DB_ERROR("DBTransactionManager::close", err);
        return err;
    }
    return 0;
}

int DBTransactionManager::checkpoint(long kbyte, long min) const
{
    int err;
    const DB_TXNMGR *mgr = unwrapConst(this);
    if ((err = txn_checkpoint(mgr, kbyte, min)) != 0) {
        DB_ERROR("DBTransactionManager::checkpoint", err);
        return err;
    }
    return 0;
}

int DBTransactionManager::isValid() const
{
    return (unwrapConst(this) != 0);
}

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            DBTransaction                           //
//                                                                    //
////////////////////////////////////////////////////////////////////////

DBTransaction DBTransaction::NoTransaction; // use default constructor

DBTransaction::DBTransaction()
:   imp_(0)
{
}

DBTransaction::DBTransaction(DBTransactionManager &manager,
                               DBTransaction nestedParent)
:   imp_(0)
{
    DB_TXNMGR *mgr = unwrap(&manager);
    DB_TXN *parent = unwrap(&nestedParent);
    DB_TXN *txn;

    if (!mgr) {
        DB_ERROR("DBTransaction::DBTransaction", EINVAL);
    }
    int err;
    if ((err = txn_begin(mgr, parent, &txn)) != 0) {
        DB_ERROR("DBTransaction::DBTransaction", err);
    }
    imp_ = wrap(txn);
}

int DBTransaction::isValid() const
{
    return (unwrapConst(this) != 0);
}

int DBTransaction::abort()
{
    int err;
    DB_TXN *txn = unwrap(this);
    if ((err = txn_abort(txn)) != 0) {
        DB_ERROR("DBTransaction::abort", err);
        return err;
    }
    return 0;
}

int DBTransaction::commit()
{
    int err;
    DB_TXN *txn = unwrap(this);
    if ((err = txn_commit(txn)) != 0) {
        DB_ERROR("DBTransaction::commit", err);
        return err;
    }
    return 0;
}

u_int32_t DBTransaction::id()
{

    int err;
    DB_TXN *txn = unwrap(this);
    if ((err = txn_id(txn)) != 0) {
        DB_ERROR("DBTransaction::id", err);
        return err;
    }

    return 0;
}

int DBTransaction::prepare()
{
    int err;
    DB_TXN *txn = unwrap(this);
    if ((err = txn_prepare(txn)) != 0) {
        DB_ERROR("DBTransaction::prepare", err);
        return err;
    }
    return 0;
}
